/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        24.10.2007
 Description:    Timer Routinen
 Modified:       G. Menke, 15.03.2009

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "timer.h"

volatile unsigned long time;
volatile unsigned long time_watchdog = 0;
volatile uint8_t hw_wdt_run_flag = 0;

//----------------------------------------------------------------------------
//Diese Routine startet und inizialisiert den Timer
void timer_init (void)
{
	TCCR1B |= (1<<WGM12) | (1<<CS10 | 0<<CS11 | 1<<CS12);
	TCNT1 = 0;
	OCR1A = (F_CPU / 1024) - 1;
	TIMSK |= (1 << OCIE1A);
	return;
};

//----------------------------------------------------------------------------
//Diese Routine gibt den Timerwert zurck
unsigned long timer_get (void) {
	return time;
}

//----------------------------------------------------------------------------
//Timer Interrupt
ISR (TIMER1_COMPA_vect)
{
	//tick 1 second
	time++;
    if((stack_watchdog++) > WTT)  //emergency reset of the stack
    {
        RESET();
	}
    eth.timer = 1;
}

//----------------------------------------------------------------------------
// if enabled, toggle the hardware wdt pin
void do_toggle_hw_wdt(void) {
	if(hw_wdt_run_flag) {
		PORTD ^= 1 << PD2;
	}
}

//----------------------------------------------------------------------------
// enable/disable the hardware wdt pin
void do_toggle_hw_wdt_enable(uint8_t enable) {
	if(enable) {
		hw_wdt_run_flag = 1;
	}
	else {
		hw_wdt_run_flag = 0;
		PORTD &= ~(1<<PD2);
	}
}
